# MSWord Document Product for Zope - MSDocument.py
# Copyright (C) 2001 ric BARROCA <eric@barroca.net>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2, or (at your option)
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
# 02111-1307, USA.
#
# MSWordDocument.py: ZClass supporting MS-Word (6/95/97/2000) documents,
#					 translating them into HTML for preview (using MSWord 
#					 module) and indexing (using ZCatalog for example).

from OFS import SimpleItem
from Globals import HTMLFile
from Globals import Persistent
from Globals import MessageDialog
from OFS.PropertyManager import PropertyManager
from OFS.ObjectManager import ObjectManager
from AccessControl.Role import RoleManager
from OFS import Image
from webdav.common import rfc1123_date
from OFS.SimpleItem import Item_w__name__
from DateTime import DateTime
from OFS.Cache import Cacheable
import string

import Acquisition, re, string, os

if os.name == 'posix':
    from MSWord import MSWordDoc
else:
    from MSWordWin import MSWordDoc


##def wdebug(msg):
##        file = open('c:\mswddebug.txt', 'a')
##        file.write(str(msg) + '\n')
##        file.close()

##Useful method to set up the id
def cookId(id, title, file):
    "Cook id and title"
    if not id and hasattr(file, 'filename'):
        filename = file.filename
        title = title or filename
        id = filename[max(string.rfind(filename, '/'),
                          string.rfind(filename, '\\'),
                          string.rfind(filename, ':'),
                          )+1:]
    return id, title

def cookName(name):
    ext = string.lower(name[-4:])
    #wdebug("cookName -- name: %s -- ext: %s" % (name, ext))
    if ext == ".doc":
        #wdebug("EXT OK")
        newname = name
    else:
        #wdebug("EXT NOT OK")
        newname = name + ".doc"
    return newname


class MSWordDocument(
    SimpleItem.Item,
    Persistent,
    Item_w__name__,
    Cacheable,
    Acquisition.Implicit,
    ObjectManager,
    PropertyManager,
    RoleManager,
    ):
    """
    MSWord Document
    """
    
    meta_type = "MSWord Document"
    
    manage_options = (
        (
        {'label': 'Edit',		'action': 'manage_main'},
        {'label': 'Upload',		'action': 'manage_uploadForm'},
        {'label': 'View',		'action': ''},
        )
        + PropertyManager.manage_options
        + RoleManager.manage_options
        + Item_w__name__.manage_options
        + Cacheable.manage_options
        )
    
    # Let's define some permissions
    __ac_permissions__=(
        ('View management screens', ('manage_tabs','manage_main')),
        ('Change MSWord Document', ('manage_edit','manage_editForm', 'manage', 'manage_main', 'manage_uploadForm',
                                    'manage_edit', 'manage_upload', 'PUT', 'manage_historyCopy',
                                    'manage_beforeHistoryCopy', 'manage_afterHistoryCopy')),
        ('View', ('index_html', 'viewHTML', 'download', 'showImg', 'manage_FTPget')),
        )

    # Two useful properties (do you see others ?)
    _properties=(
        {'id':'title', 'type': 'string'},
        {'id':'name', 'type':'string'},
        ) 

    def __init__(self, id, file, title=''):
        "Initialiaze a new instance"
        #wdebug("INIT")
        #wdebug("id: %s -- type file: %s" % (id, type(file)))
        self.__name__ = id
        self.id = id
        self.title = title
        if type(file) is not type(''):
            #wdebug("TYPE FILE")
            data = file.read()
            name, title = cookId("", "", file)
        else:
            #wdebug("TYPE STRING")
            data = file
            name = id
        
        self.update_data(data, name)
        #wdebug("INIT END -- END UDPATE")
        
    manage_addForm = HTMLFile('addForm', globals())

    def manage_add(self, id, file, title='', REQUEST=None):
        "Add a MSWord Document to a folder."
        #wdebug("manage_add")
        id, title = cookId(id, title, file)
        self._setObject(id, MSWordDocument(id, file, title))
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url()+'/manage_main')
            
    viewHTML = HTMLFile('index', globals())

    manage_main = HTMLFile('editForm', globals())

    def manage_edit(self, title, REQUEST=None):
        "Edit"
        self.title = title
        return MessageDialog(
            title = "Edited",
            message = "Properties for %s changed" % self.id,
            action = "manage_main",
            )

    manage_uploadForm = HTMLFile('uploadForm', globals())

    def manage_upload(self, file, REQUEST=None):
        "Upload the MSWord Document and process it (update_data)"
        #wdebug('UPLOAD %s' % self.id)
        if type(file) is not type(''):
            #wdebug('FILE TYPE: FILE')
            data = file.read()
            name, title = cookId("", "", file)
        else:
            #wdebug('FILE TYPE: %s' % type(data))
            data = file
            name = self.name
        #wdebug("FILENAME: %s" % name)
        self.update_data(data, name)
                
        return MessageDialog(
            title = "Uploaded",
            message = "MSWord Document data for %s uploaded" % self.id,
            action = "manage_main",
            )
            
    def update_data(self, data, name):
        "Process the MSWord Document"
        #wdebug("UPDATE_DATA -- name: %s -- size: %s" % (name, len(data)))
        self.size = len(data)
        self.name = name
        #Clean this object (delete images and msword file)
        #self.clean()
        
        #Let's convert the word document into html
        doc = MSWordDoc(self.name, data)
        doc.Convert()
        html = doc.getDocHTML()
        path, imgs = doc.getImgs()
        self.text = html
        #wdebug("LEN HTML %s" % len(html))
        self.imgs = imgs
        #wdebug("IMGS LIST %s" % imgs)
        #wdebug("END CONVERT")
        #wdebug("START IMPORT IMGS -- path: %s" % path)
        self.importImgs(path)
        #wdebug("END IMPORT IMGS")
        #wdebug("START IMPORT DOC -- lan data: %s" % len(data))
        self.importDoc(data)
        #wdebug("END IMPORT DOC")
        doc.cleandir()
        self.ZCacheable_invalidate()
    
##    def clean(self):
##        wdebug("CLEANDOC START")
##        for i in self.objectIds():
##            wdebug("DELETE %s" % i)
##            self.manage_delObjects(i)
##        wdebug("CLEANDOC END")
##        return 1

    def importDoc(self, data):
        if hasattr(self, 'docfile'): self.manage_delObjects('docfile')
        self.manage_addFile('docfile', data, content_type="application/msword")
        return 1

    def importImgs(self, path):
        for imgname in self.imgs:
            #wdebug("IMPORT IMG %s : %s%s" % (imgname, path, imgname))
            if hasattr(self, imgname): self.manage_delObjects(imgname)
            #wdebug("OPEN IMG")
            file = open(path + imgname, 'rb')
            #wdebug("START READING IMG")
            data = file.read()
            file.close()
            #wdebug("END READING IMG")
            #wdebug("START ADDING IMAGE")
            self.manage_addImage(imgname, data)
            #wdebug("END ADDING IMAGE")
        return 1
    
    def download(self, REQUEST, RESPONSE):
        "Send the original msword file"
        RESPONSE.redirect(self.absolute_url()+'/docfile')
        return ''
    
    def index_html(self, REQUEST=None, RESPONSE=None):
        "Send the MSWord Document"
        if re.match('.*dav.*', REQUEST['HTTP_USER_AGENT']): 
            self.download(RESPONSE)
        else:
            RESPONSE.redirect(self.absolute_url()+'/viewHTML')
            return ''

######
## Protocols handlers
###############
    def PUT(self, REQUEST, RESPONSE):
        """Handle HTTP PUT requests"""
        #wdebug("PUT")
        self.dav__init(REQUEST, RESPONSE)
        file=REQUEST['BODYFILE']
        data = file.read()
        #wdebug("PUT - END FILE READ")
        name = cookName(self.id)
        #wdebug("PUT - cookName")
        #wdebug("FILENAME: %s" % name)
        self.update_data(data, name)
        
        RESPONSE.setStatus(204)
        return RESPONSE

    #Send the MSWord document via FTP
    def manage_FTPget(self):
        #wdebug("FTP GET !!")
        self.docfile.manage_FTPget()

    #Useful method :-) for DAV and FTP clients
    def get_size(self):
        #wdebug("GET SIZE")
        return self.size
    
    def getContentType(self):
        return "application/msword"


